import sys
import random
from string import ascii_lowercase

from .cli import parse_args
from .utils import WordListManager, has_repeating_letters
from .words import word_list


def run(num_words=3, unique_letters=None):
    used_letters = set()
    selected_words = []
    
    # Filter word list if unique letters flag is set
    available_words = word_list
    if unique_letters is not None:
        available_words = [word for word in word_list if not has_repeating_letters(word)]
    for word in random.sample(available_words, len(available_words)):
        if not any(letter in used_letters for letter in word):
            selected_words.append(word)
            used_letters.update(word)

        if len(selected_words) == num_words:
            break

    used_letters = "".join(
        letter if letter in used_letters else "_" for letter in ascii_lowercase
    )

    print("Selected words:", selected_words)
    print("Used letters:", used_letters.upper())


def main():
    # Handle special case: positional number argument before flags
    # Transform 'ww 3 -u' into 'ww -n 3 -u' for argparse
    if len(sys.argv) > 1 and sys.argv[1].isdigit():
        num = sys.argv[1]
        sys.argv = [sys.argv[0]] + ['-n', num] + sys.argv[2:]
    
    args = parse_args()
    
    # If no action specified, run word selection
    if not args.action:
        num_words = args.num_words
        run(num_words, unique_letters=args.u)
        return
    
    # Handle management actions
    manager = WordListManager()
    
    match args.action:
        case "stats":
            manager.show_stats()
        case "find-scarce":
            manager.find_scarce_letters(args.num)
        case "dedup":
            manager.remove_duplicates()
        case "sort":
            manager.sort_words()
        case "add":
            if not args.word:
                print("Error: No word provided to add.")
                sys.exit(1)
            manager.add_word(args.word)
        case "clean":
            manager.remove_invalid_words()
            print("Clean operation completed")
        case _:
            num_words = args.num_words
            run(num_words)



word_list = [
 'aback', 'abase', 'abate', 'abhor', 'abode', 'abort', 'about', 'above', 'abuse', 'abyss',
 'ached', 'acids', 'acorn', 'acres', 'acrid', 'actin', 'actor', 'acute', 'adage', 'adapt',
 'added', 'adder', 'addle', 'adept', 'adieu', 'adlib', 'admin', 'admit', 'adobe', 'adopt',
 'adore', 'adorn', 'adult', 'aeons', 'affix', 'afire', 'afoot', 'afore', 'afoul', 'after',
 'again', 'agape', 'agate', 'agent', 'agile', 'aging', 'aglow', 'agony', 'agree', 'ahead',
 'aided', 'aider', 'aimer', 'aisle', 'alamo', 'alarm', 'album', 'alder', 'aleph', 'alert',
 'algae', 'algal', 'algas', 'alias', 'alibi', 'alien', 'align', 'alike', 'alive', 'allay',
 'alley', 'allot', 'allow', 'alloy', 'aloes', 'aloft', 'alone', 'along', 'aloof', 'aloud',
 'alpha', 'altar', 'alter', 'altos', 'amass', 'amaze', 'amber', 'amble', 'amend', 'amino',
 'amiss', 'amity', 'among', 'amped', 'ample', 'amply', 'amuse', 'angel', 'anger', 'angle',
 'angry', 'angst', 'anile', 'anime', 'anise', 'ankle', 'annex', 'annoy', 'annul', 'anode',
 'antic', 'anvil', 'aorta', 'apace', 'apart', 'aphid', 'aping', 'apnea', 'apple', 'apply',
 'apron', 'arbor', 'arced', 'ardor', 'areas', 'arena', 'argue', 'arise', 'armed', 'armor',
 'aroma', 'arose', 'arras', 'array', 'arrow', 'arson', 'artsy', 'ascot', 'ashen', 'asian',
 'aside', 'asked', 'askew', 'aspen', 'assay', 'asses', 'asset', 'aster', 'atlas', 'atoll',
 'atoms', 'atone', 'attic', 'audio', 'audit', 'auger', 'aught', 'augur', 'aunty', 'aural',
 'autos', 'avail', 'avant', 'avert', 'avian', 'avoid', 'await', 'awake', 'award', 'aware',
 'awash', 'awful', 'awoke', 'axiom', 'axion', 'azure', 'bacon', 'badge', 'badly', 'bagel',
 'baker', 'baler', 'balky', 'balmy', 'banal', 'banjo', 'barge', 'barmy', 'baron', 'basal',
 'based', 'basic', 'basil', 'basin', 'basis', 'baste', 'batch', 'bathe', 'beach', 'beads',
 'beady', 'beard', 'bears', 'beast', 'beats', 'beech', 'beefy', 'befit', 'began', 'begat',
 'beget', 'begin', 'begun', 'being', 'belch', 'belie', 'belle', 'belly', 'below', 'bench',
 'beret', 'berry', 'berth', 'beset', 'betas', 'bevel', 'bezel', 'bicep', 'biddy', 'bigot',
 'bilge', 'billy', 'binge', 'bingo', 'biome', 'birch', 'birds', 'birth', 'bison', 'biter',
 'bites', 'black', 'blade', 'blame', 'bland', 'blank', 'blare', 'blast', 'blaze', 'bleak',
 'bleat', 'blend', 'bless', 'blimp', 'blind', 'blink', 'blitz', 'block', 'blood', 'bloom',
 'blown', 'blues', 'bluff', 'blunt', 'blush', 'board', 'boats', 'bobby', 'bones', 'bongo',
 'books', 'boost', 'booth', 'boots', 'borne', 'bound', 'boxes', 'brace', 'brain', 'brake',
 'brand', 'brash', 'brave', 'bravo', 'bread', 'break', 'breed', 'briar', 'brick', 'brief',
 'brine', 'bring', 'brink', 'briny', 'brisk', 'broad', 'broil', 'broke', 'brood', 'brown',
 'brunt', 'brush', 'build', 'built', 'bumpy', 'burnt', 'buyer', 'buzzy', 'bylaw', 'cabal',
 'cable', 'cacao', 'cache', 'cacti', 'caddy', 'cadet', 'cagey', 'cairn', 'calif', 'calls',
 'camel', 'cameo', 'camps', 'canal', 'candy', 'canoe', 'canon', 'caper', 'caput', 'carat',
 'cards', 'carey', 'cargo', 'carlo', 'carol', 'carry', 'carte', 'carve', 'cases', 'caste',
 'casts', 'catch', 'cater', 'catty', 'caulk', 'cause', 'cells', 'cents', 'chafe', 'chain',
 'chair', 'champ', 'chant', 'chaos', 'chard', 'charm', 'chart', 'chase', 'chasm', 'cheap',
 'cheat', 'check', 'cheek', 'cheer', 'chess', 'chest', 'chick', 'chide', 'chief', 'child',
 'chili', 'chill', 'chime', 'chimp', 'china', 'chirp', 'chock', 'choir', 'choke', 'chomp',
 'chord', 'chore', 'chose', 'chris', 'chuck', 'chump', 'chunk', 'churn', 'chute', 'cider',
 'cigar', 'cinch', 'circa', 'civic', 'civil', 'clack', 'claim', 'clamp', 'clash', 'class',
 'clave', 'clean', 'clear', 'cleft', 'click', 'cliff', 'climb', 'cling', 'clink', 'clock',
 'clomp', 'close', 'cloth', 'cloud', 'clout', 'coach', 'coast', 'cocoa', 'codes', 'coins',
 'color', 'combo', 'comes', 'comfy', 'comic', 'comma', 'conch', 'conga', 'coopt', 'coral',
 'corps', 'costs', 'couch', 'cough', 'could', 'count', 'court', 'couth', 'coven', 'coyly',
 'craft', 'cramp', 'crane', 'crank', 'crash', 'crass', 'crate', 'crave', 'cream', 'creek',
 'creme', 'crick', 'crime', 'crimp', 'crisp', 'croup', 'crowd', 'crown', 'crude', 'crumb',
 'crush', 'crypt', 'cubit', 'cumin', 'cupid', 'curse', 'curso', 'curve', 'cycle', 'cynic',
 'daily', 'dairy', 'daisy', 'dance', 'dated', 'daunt', 'dazed', 'deals', 'dealt', 'death',
 'debar', 'debit', 'debut', 'decaf', 'decay', 'decor', 'decoy', 'decry', 'dedup', 'defer',
 'delay', 'delta', 'demon', 'demur', 'denim', 'dense', 'depal', 'depot', 'depth', 'derby',
 'deter', 'detox', 'deuce', 'devil', 'diary', 'diced', 'digit', 'dilly', 'dimly', 'diner',
 'dingo', 'dingy', 'diode', 'dirge', 'dirty', 'disco', 'ditch', 'ditto', 'ditty', 'diver',
 'dizzy', 'dodge', 'dodgy', 'doggy', 'dogma', 'doing', 'donor', 'doors', 'doubt', 'dowel',
 'dozed', 'dozen', 'draft', 'drama', 'drank', 'drape', 'drawn', 'dread', 'dream', 'dress',
 'dried', 'drill', 'drink', 'drive', 'droll', 'drone', 'drops', 'drove', 'drown', 'drugs',
 'druid', 'drums', 'drunk', 'ducks', 'dully', 'dummy', 'dusty', 'dwarf', 'dweeb', 'dying',
 'eager', 'eagle', 'early', 'earst', 'earth', 'easel', 'eaten', 'eater', 'ebony', 'edges',
 'edict', 'edify', 'eerie', 'egret', 'eight', 'eject', 'elbow', 'elder', 'elect', 'elegy',
 'elfin', 'elide', 'elite', 'elope', 'elude', 'email', 'embed', 'ember', 'emcee', 'empty',
 'enact', 'endow', 'enema', 'enemy', 'enjoy', 'ennui', 'ensue', 'enter', 'entry', 'envoy',
 'epoch', 'epoxy', 'equal', 'equip', 'erase', 'erect', 'erode', 'error', 'erupt', 'essay',
 'ester', 'ether', 'ethic', 'evade', 'evans', 'event', 'every', 'evict', 'evoke', 'exact',
 'exalt', 'excel', 'exert', 'exile', 'exist', 'exits', 'expel', 'extol', 'extra', 'fable',
 'faced', 'faces', 'facet', 'facts', 'fails', 'faint', 'fairy', 'faith', 'falls', 'false',
 'fancy', 'farce', 'farms', 'fatal', 'fatty', 'fault', 'fauna', 'favor', 'feast', 'fecal',
 'feign', 'femme', 'femur', 'fence', 'fetal', 'fetch', 'fetid', 'fever', 'fewer', 'fiber',
 'ficus', 'field', 'fiend', 'fifer', 'fifth', 'fifty', 'fight', 'filed', 'files', 'filet',
 'filly', 'films', 'filmy', 'filth', 'final', 'finch', 'finer', 'fires', 'firms', 'first',
 'fishy', 'fixed', 'fixer', 'fizzy', 'fjord', 'flack', 'flags', 'flail', 'flair', 'flake',
 'flame', 'flank', 'flare', 'flash', 'flask', 'fleck', 'fleet', 'flesh', 'flick', 'flier',
 'flies', 'fling', 'flint', 'flirt', 'float', 'flock', 'flood', 'floor', 'flora', 'floss',
 'flour', 'flout', 'flown', 'flows', 'fluff', 'fluid', 'fluke', 'flume', 'flush', 'flute',
 'foamy', 'focal', 'focus', 'foggy', 'foist', 'folio', 'folks', 'folly', 'fonts', 'foods',
 'foray', 'force', 'forge', 'forgo', 'forms', 'forte', 'forth', 'forty', 'forum', 'found',
 'foyer', 'frail', 'frame', 'frank', 'fraud', 'freak', 'freed', 'freer', 'fresh', 'friar',
 'fried', 'frill', 'frisk', 'fritz', 'frock', 'front', 'frost', 'froth', 'frown', 'froze',
 'fruit', 'fudge', 'fugue', 'fully', 'funds', 'fungi', 'funky', 'funny', 'furry', 'fussy',
 'fuzzy', 'gamer', 'gamma', 'gassy', 'gates', 'gauge', 'gavel', 'gawky', 'gazer', 'gecko',
 'geeky', 'geese', 'genes', 'genre', 'ghost', 'ghoul', 'giant', 'giddy', 'gifts', 'gipsy',
 'girls', 'girly', 'given', 'giver', 'gives', 'gizmo', 'glade', 'gland', 'glare', 'glass',
 'glaze', 'gleam', 'glean', 'glide', 'glint', 'glitz', 'gloat', 'globe', 'gloom', 'glory',
 'gloss', 'glove', 'glyph', 'gnash', 'gnome', 'goals', 'godly', 'going', 'golem', 'golly',
 'gonad', 'goods', 'goody', 'gooey', 'goose', 'gorge', 'gouge', 'grace', 'grade', 'graft',
 'grail', 'grain', 'grand', 'grant', 'grape', 'graph', 'grasp', 'grass', 'grate', 'grave',
 'gravy', 'great', 'greed', 'green', 'greet', 'grief', 'grift', 'grill', 'grime', 'grimy',
 'grind', 'gripe', 'groan', 'groin', 'groom', 'grope', 'gross', 'group', 'grout', 'grove',
 'growl', 'grown', 'grows', 'gruff', 'grump', 'grunt', 'guano', 'guard', 'guava', 'guess',
 'guest', 'guide', 'guild', 'guile', 'guilt', 'guise', 'gulch', 'gully', 'gumbo', 'guppy',
 'gusto', 'gusty', 'gypsy', 'habit', 'hairy', 'halve', 'hands', 'handy', 'hangs', 'happy',
 'hardy', 'harem', 'harpy', 'harry', 'harsh', 'haste', 'hasty', 'hatch', 'hater', 'haunt',
 'haven', 'hazel', 'heads', 'heady', 'heard', 'heart', 'heath', 'heave', 'heavy', 'hedge',
 'hedon', 'hefty', 'heist', 'helen', 'helix', 'hello', 'helps', 'hence', 'henry', 'heron',
 'hills', 'hilly', 'hinge', 'hints', 'hippy', 'hired', 'hitch', 'hoard', 'hobby', 'hoist',
 'holds', 'holes', 'holly', 'homer', 'homes', 'honey', 'honor', 'hoped', 'hopes', 'horde',
 'horny', 'horse', 'hosts', 'hotel', 'hound', 'hours', 'house', 'hovel', 'hover', 'howdy',
 'huffy', 'human', 'humid', 'humor', 'humph', 'hunch', 'hunky', 'hurry', 'husky', 'hyena',
 'hymen', 'icily', 'icing', 'ideal', 'ideas', 'image', 'inane', 'inbox', 'index', 'india',
 'inert', 'inlay', 'inner', 'input', 'irony', 'issue', 'items', 'ivory', 'jaded', 'james',
 'janky', 'japan', 'jazzy', 'jeans', 'jelly', 'jenny', 'jerry', 'jesse', 'jewel', 'jiffy',
 'jimmy', 'jived', 'johns', 'joins', 'joint', 'joker', 'jolly', 'jones', 'joust', 'judge',
 'judgy', 'juice', 'jumbo', 'jumpy', 'junky', 'kappa', 'karma', 'kayak', 'keeps', 'kelly',
 'kings', 'kiosk', 'knack', 'known', 'knows', 'koala', 'label', 'lacks', 'ladle', 'lakes',
 'lance', 'lands', 'lanes', 'laped', 'lapse', 'large', 'laser', 'latch', 'later', 'lathe',
 'laugh', 'laura', 'layer', 'leach', 'leads', 'leapt', 'learn', 'lease', 'leash', 'least',
 'leave', 'leeds', 'leery', 'lefty', 'legal', 'lemon', 'lemur', 'level', 'lever', 'lewis',
 'libel', 'light', 'liked', 'liken', 'likes', 'lilac', 'limbo', 'limit', 'lined', 'lines',
 'lingo', 'links', 'lions', 'lists', 'lithe', 'lived', 'liver', 'lives', 'livid', 'loads',
 'loans', 'lobby', 'local', 'locks', 'lodge', 'logic', 'logos', 'looks', 'loops', 'loopy',
 'loose', 'lords', 'loser', 'loses', 'lousy', 'loved', 'lover', 'loves', 'lower', 'loyal',
 'lucid', 'lucky', 'lumen', 'lunar', 'lunch', 'lurid', 'lying', 'macho', 'macro', 'magic',
 'magma', 'major', 'maker', 'makes', 'males', 'mambo', 'mango', 'manor', 'maple', 'march',
 'maria', 'marie', 'marsh', 'mason', 'masse', 'match', 'mates', 'maths', 'maybe', 'mayor',
 'meals', 'means', 'meant', 'meaty', 'media', 'meets', 'melon', 'menus', 'mercy', 'merge',
 'merit', 'merry', 'messy', 'metal', 'meter', 'metro', 'micro', 'midst', 'might', 'miles',
 'mills', 'mimic', 'mimsy', 'minds', 'mines', 'minor', 'minus', 'mirth', 'miser', 'mixed',
 'mixer', 'mocha', 'modal', 'model', 'modes', 'money', 'month', 'moore', 'moral', 'mossy',
 'motel', 'mothy', 'motto', 'moult', 'mound', 'mount', 'mouse', 'mouth', 'moved', 'moves',
 'movie', 'mucky', 'munch', 'mushy', 'music', 'myrrh', 'nadir', 'named', 'nanny', 'nappy',
 'naval', 'navel', 'needs', 'nerve', 'never', 'newer', 'newly', 'niche', 'night', 'ninja',
 'noble', 'nodes', 'noise', 'north', 'noted', 'notes', 'notre', 'novel', 'nudge', 'nurse',
 'nylon', 'nymph', 'oaken', 'oasis', 'obese', 'occur', 'ocean', 'octal', 'octet', 'offer',
 'often', 'olden', 'older', 'olive', 'omega', 'oomph', 'opens', 'opera', 'opine', 'optic',
 'orate', 'orbit', 'order', 'organ', 'other', 'otter', 'ought', 'outer', 'ovale', 'owned',
 'owner', 'oxide', 'pacer', 'packs', 'paddy', 'pages', 'paint', 'pairs', 'paled', 'palsy',
 'panda', 'panel', 'panic', 'pants', 'paper', 'parch', 'pares', 'parks', 'parse', 'parts',
 'party', 'pasta', 'paste', 'patch', 'paths', 'patio', 'patsy', 'pause', 'peace', 'peach',
 'peaks', 'pearl', 'pears', 'pedal', 'peers', 'penny', 'perch', 'pesto', 'petal', 'peter',
 'phase', 'phlox', 'phone', 'photo', 'piano', 'picks', 'piece', 'piles', 'pills', 'pilot',
 'pinky', 'pinot', 'pipes', 'pique', 'pitch', 'pithy', 'pivot', 'pixel', 'pixie', 'pizza',
 'place', 'plaid', 'plain', 'plane', 'plank', 'plans', 'plant', 'plate', 'plays', 'plaza',
 'plead', 'pleat', 'plier', 'plots', 'pluck', 'plumb', 'plume', 'plunk', 'plush', 'poems',
 'point', 'poker', 'polar', 'poles', 'polls', 'polyp', 'pools', 'poppy', 'porch', 'ports',
 'posts', 'pouch', 'pound', 'power', 'prank', 'press', 'price', 'pride', 'prime', 'print',
 'prior', 'prism', 'privy', 'prize', 'probe', 'prone', 'proof', 'prose', 'proud', 'prove',
 'proxy', 'prude', 'pudgy', 'puffy', 'pulls', 'pulse', 'pumps', 'punch', 'pupil', 'puppy',
 'purse', 'pushy', 'pylon', 'quack', 'quaff', 'quail', 'quake', 'quant', 'quark', 'quart',
 'quash', 'queen', 'query', 'quest', 'queue', 'quick', 'quiet', 'quill', 'quilt', 'quint',
 'quirk', 'quite', 'quote', 'rabid', 'racer', 'radar', 'radii', 'radio', 'raise', 'rally',
 'ranch', 'range', 'ranks', 'rapid', 'rated', 'rates', 'ratio', 'raven', 'reach', 'react',
 'reads', 'ready', 'realm', 'reaps', 'rebar', 'rebel', 'redux', 'refer', 'refit', 'regal',
 'relax', 'relay', 'relic', 'remit', 'repay', 'repel', 'reply', 'reset', 'resin', 'retro',
 'rhino', 'rhyme', 'rider', 'rides', 'ridge', 'rifle', 'right', 'rings', 'risen', 'rises',
 'risks', 'river', 'rivet', 'roads', 'robin', 'rocks', 'rocky', 'roger', 'rogue', 'roles',
 'rolls', 'roman', 'rooms', 'roots', 'roped', 'roses', 'rouge', 'rough', 'round', 'rouse',
 'route', 'royal', 'rugby', 'ruled', 'ruler', 'rules', 'rumor', 'runny', 'rural', 'saber',
 'sable', 'safer', 'salad', 'sales', 'salon', 'salsa', 'salty', 'sands', 'sandy', 'santa',
 'sassy', 'satin', 'sauce', 'savor', 'scale', 'scant', 'scare', 'scary', 'scene', 'scoff',
 'scone', 'scope', 'score', 'scout', 'scrap', 'screw', 'seats', 'seeds', 'seedy', 'seeks',
 'seems', 'segue', 'sells', 'sends', 'sense', 'serve', 'setup', 'seven', 'sever', 'shade',
 'shady', 'shaft', 'shake', 'shall', 'shame', 'shank', 'shape', 'shard', 'share', 'shark',
 'sharp', 'shave', 'shear', 'sheep', 'sheet', 'shelf', 'shell', 'shiek', 'shift', 'shine',
 'ships', 'shirt', 'shock', 'shoes', 'shone', 'shoot', 'shops', 'shore', 'shorn', 'short',
 'shout', 'shown', 'shows', 'shrew', 'shuck', 'sides', 'siege', 'sight', 'sigma', 'signs',
 'silly', 'simon', 'since', 'sinch', 'sissy', 'sites', 'sixth', 'sized', 'sizes', 'skill',
 'skins', 'skirt', 'skunk', 'slant', 'slate', 'slave', 'sleek', 'sleep', 'slice', 'slick',
 'slide', 'slime', 'slimy', 'sling', 'sloop', 'slope', 'slosh', 'slots', 'slump', 'slung',
 'smack', 'small', 'smart', 'smash', 'smelt', 'smile', 'smirk', 'smith', 'smoke', 'snack',
 'snake', 'snaky', 'sneak', 'snide', 'snoop', 'snout', 'solar', 'solid', 'solve', 'songs',
 'sonic', 'sorry', 'sorts', 'souls', 'sound', 'south', 'space', 'spade', 'spank', 'spare',
 'spark', 'spasm', 'speak', 'spear', 'speed', 'spend', 'spent', 'spike', 'spilt', 'spine',
 'spiny', 'spire', 'splat', 'splay', 'split', 'spoil', 'spoke', 'spoof', 'spook', 'spool',
 'spore', 'spork', 'sport', 'spots', 'spray', 'spunk', 'spurt', 'squad', 'stack', 'staff',
 'stage', 'stake', 'stale', 'stalk', 'stall', 'stamp', 'stand', 'stank', 'stare', 'stark',
 'stars', 'start', 'stash', 'state', 'stays', 'steal', 'steam', 'steel', 'steep', 'steps',
 'stern', 'steve', 'stick', 'still', 'stilt', 'stink', 'stock', 'stomp', 'stone', 'stood',
 'stool', 'stops', 'store', 'stork', 'storm', 'story', 'stout', 'stove', 'strap', 'stray',
 'strip', 'strut', 'stuck', 'study', 'stuff', 'stunk', 'stunt', 'style', 'sugar', 'suite',
 'suits', 'sunny', 'super', 'surge', 'susan', 'sushi', 'swami', 'swamp', 'swank', 'swarm',
 'swath', 'sweat', 'sweep', 'sweet', 'swell', 'swept', 'swift', 'swing', 'swipe', 'swiss',
 'swoop', 'sword', 'sworn', 'syrup', 'table', 'taboo', 'tacet', 'tacit', 'taken', 'takes',
 'talep', 'tales', 'talks', 'tango', 'tangy', 'tanks', 'taper', 'tapes', 'tapir', 'tarot',
 'tarpa', 'tarry', 'taser', 'tasks', 'taste', 'tasty', 'tatty', 'taunt', 'taupe', 'taxes',
 'teach', 'teams', 'tears', 'teary', 'teeth', 'telex', 'tempo', 'tenet', 'tenor', 'tense',
 'tepee', 'tepid', 'terry', 'terse', 'tests', 'tetra', 'texas', 'texts', 'thank', 'theca',
 'theft', 'their', 'theme', 'there', 'these', 'theta', 'thick', 'thief', 'thine', 'thing',
 'think', 'third', 'thong', 'those', 'three', 'threw', 'throw', 'thrum', 'thumb', 'thump',
 'thyme', 'tibia', 'tidal', 'tiger', 'tight', 'tilde', 'tiles', 'timer', 'times', 'timid',
 'tired', 'tithe', 'title', 'tizzy', 'toady', 'toast', 'today', 'token', 'tommy', 'tonal',
 'tones', 'tonic', 'tools', 'tooth', 'topaz', 'topic', 'torus', 'total', 'touch', 'tough',
 'tours', 'towel', 'tower', 'towns', 'toxic', 'toxin', 'trace', 'track', 'tract', 'tracy',
 'trade', 'trail', 'train', 'trait', 'tramp', 'trans', 'traps', 'trash', 'treat', 'treed',
 'trees', 'trend', 'trial', 'tribe', 'trice', 'trick', 'tried', 'tries', 'trims', 'tripe',
 'trips', 'trite', 'troll', 'troop', 'trove', 'truce', 'truck', 'truly', 'trump', 'trunk',
 'trust', 'truth', 'tryst', 'tuber', 'tubes', 'tudor', 'tulip', 'tulle', 'tulsa', 'tummy',
 'tumor', 'tuned', 'tuner', 'tunic', 'turbo', 'turns', 'tutor', 'tweak', 'tweet', 'twice',
 'twine', 'twins', 'twirl', 'twist', 'tying', 'typed', 'types', 'ultra', 'umbra', 'uncle',
 'under', 'undue', 'union', 'unite', 'units', 'unity', 'untap', 'until', 'unzip', 'upend',
 'upper', 'upset', 'urban', 'urged', 'usage', 'users', 'usher', 'using', 'usual', 'utero',
 'utils', 'utter', 'uvula', 'vague', 'valet', 'valid', 'valor', 'value', 'valve', 'vapid',
 'vapor', 'vault', 'venom', 'venue', 'verde', 'verge', 'verse', 'verve', 'video', 'views',
 'vigor', 'villa', 'vinyl', 'viola', 'virus', 'visit', 'vista', 'vital', 'vivid', 'vixen',
 'vocal', 'voice', 'voila', 'vomit', 'voter', 'votes', 'vouch', 'vowel', 'wages', 'wagon',
 'waist', 'wales', 'walks', 'walls', 'waltz', 'wants', 'waste', 'watch', 'water', 'waves',
 'wayne', 'weary', 'weeks', 'weigh', 'weird', 'wells', 'welsh', 'wench', 'whale', 'wharf',
 'wheat', 'wheel', 'where', 'which', 'whiff', 'while', 'whisk', 'white', 'whole', 'whose',
 'wider', 'widow', 'width', 'wills', 'wines', 'wings', 'wired', 'wires', 'wiser', 'witch',
 'witty', 'woman', 'women', 'woods', 'woody', 'words', 'works', 'world', 'worry', 'worse',
 'worst', 'worth', 'would', 'wound', 'wrath', 'wreak', 'wreck', 'wrist', 'write', 'wrong',
 'wrote', 'xenon', 'yacht', 'yards', 'years', 'yeast', 'yield', 'yogic', 'young', 'yours',
 'youth', 'yummy', 'zappy', 'zebra', 'zesty', 'zilch', 'zippy', 'zonal', 'zoner', 'zones',
]



import pytest
from unittest.mock import MagicMock

from src.wordle_manager.main import main
from src.wordle_manager.utils import WordListManager, has_repeating_letters
from src.wordle_manager import words


@pytest.fixture(autouse=True)
def safe_testing_environment():
    """Fixture to prevent any file writes and preserve original word list"""
    original_words = words.word_list.copy()
    original_id = id(words.word_list)

    with pytest.MonkeyPatch.context() as monkeypatch:
        monkeypatch.setattr(
            WordListManager, "save_to_file", lambda *args, **kwargs: None
        )
        try:
            yield
        finally:
            if id(words.word_list) != original_id:
                words.word_list = original_words
            else:
                words.word_list[:] = original_words


@pytest.fixture
def mock_word_list_manager():
    mock_manager = MagicMock()
    return mock_manager


@pytest.fixture
def mock_parse_args():
    mock_args = MagicMock()
    return mock_args


@pytest.mark.parametrize(
    "sys_argv, expected_action, setup_mocks, verify_calls",
    [
        pytest.param(
            ["script_name", "5"],
            "run",
            lambda: None,  # No additional setup needed
            lambda mock_run, mock_manager: mock_run.assert_called_once_with(5, unique_letters=False),
            id="run_with_number"
        ),
        pytest.param(
            ["script_name", "stats"],
            "stats",
            lambda: MagicMock(action="stats"),
            lambda mock_run, mock_manager: mock_manager.show_stats.assert_called_once(),
            id="stats_action",
        ),
        pytest.param(
            ["script_name", "find-scarce", "--num", "5"],
            "find-scarce",
            lambda: MagicMock(action="find-scarce", num=5),
            lambda mock_run,
            mock_manager: mock_manager.find_scarce_letters.assert_called_once_with(5),
            id="find_scarce_action",
        ),
        pytest.param(
            ["script_name", "dedup"],
            "dedup",
            lambda: MagicMock(action="dedup"),
            lambda mock_run,
            mock_manager: mock_manager.remove_duplicates.assert_called_once(),
            id="dedup_action",
        ),
        pytest.param(
            ["script_name", "sort"],
            "sort",
            lambda: MagicMock(action="sort"),
            lambda mock_run, mock_manager: mock_manager.sort_words.assert_called_once(),
            id="sort_action",
        ),
        pytest.param(
            ["script_name", "add", "newword"],
            "add",
            lambda: MagicMock(action="add", word="newword"),
            lambda mock_run,
            mock_manager: mock_manager.add_word.assert_called_once_with("newword"),
            id="add_action",
        ),
    ],
)
def test_main_actions(
    sys_argv, expected_action, setup_mocks, verify_calls, monkeypatch
):
    mock_word_list_manager = MagicMock()
    mock_run = MagicMock()

    # Mock the parse_args function if it's not a "run" action
    if expected_action != "run":
        mock_parse_args_result = setup_mocks()
        monkeypatch.setattr(
            "src.wordle_manager.main.parse_args", lambda: mock_parse_args_result
        )
        monkeypatch.setattr(
            "src.wordle_manager.main.WordListManager", lambda: mock_word_list_manager
        )
    else:
        # For "run" action, mock the run function
        monkeypatch.setattr("src.wordle_manager.main.run", mock_run)

    monkeypatch.setattr("sys.argv", sys_argv)

    # All actions now just return (no SystemExit)
    main()

    verify_calls(mock_run, mock_word_list_manager)


def test_main_add_action_no_word_exits_with_error(mock_parse_args, capsys, monkeypatch):
    mock_parse_args.action = "add"
    mock_parse_args.word = None
    monkeypatch.setattr("src.wordle_manager.main.parse_args", lambda: mock_parse_args)
    # Ensure sys.argv doesn't trigger the run() branch
    monkeypatch.setattr("sys.argv", ["script", "add"])

    with pytest.raises(SystemExit) as exc_info:
        main()

    assert exc_info.value.code == 1

    captured = capsys.readouterr()
    assert "Error: No word provided to add." in captured.out


def test_main_clean_action(
    mock_word_list_manager, mock_parse_args, capsys, monkeypatch
):
    mock_parse_args.action = "clean"
    monkeypatch.setattr("src.wordle_manager.main.parse_args", lambda: mock_parse_args)
    monkeypatch.setattr(
        "src.wordle_manager.main.WordListManager", lambda: mock_word_list_manager
    )
    # Ensure sys.argv doesn't trigger the run() branch
    monkeypatch.setattr("sys.argv", ["script", "clean"])

    main()

    mock_word_list_manager.remove_invalid_words.assert_called_once()

    captured = capsys.readouterr()
    assert "Clean operation completed" in captured.out


def test_main_unknown_action_with_non_numeric_word_arg(mock_parse_args, monkeypatch):
    mock_parse_args.action = "unknown"
    mock_parse_args.word = "notanumber"
    monkeypatch.setattr("src.wordle_manager.main.parse_args", lambda: mock_parse_args)
    # Ensure sys.argv doesn't trigger the run() branch
    monkeypatch.setattr("sys.argv", ["script", "unknown", "notanumber"])

    main()


def test_main_unique_letters_flag(monkeypatch):
    mock_run = MagicMock()
    monkeypatch.setattr("src.wordle_manager.main.run", mock_run)
    monkeypatch.setattr("sys.argv", ["script_name", "-u"])

    main()

    mock_run.assert_called_once_with(3, unique_letters=True)

def test_main_numeric_arg_with_unique_letters_flag(monkeypatch):
    mock_run = MagicMock()
    monkeypatch.setattr("src.wordle_manager.main.run", mock_run)
    monkeypatch.setattr("sys.argv", ["script_name", "2", "-u"])

    main()

    mock_run.assert_called_once_with(2, unique_letters=True)

@pytest.mark.parametrize("word, expected", [
    ("spill", True),  # 'l' repeats
    ("ladle", True),  # 'l' repeats
    ("apple", True),  # 'p' repeats
    ("crumb", False),
    ("brown", False),
    ("mango", False),
    ("spicy", False),
])
def test_has_repeating_letters(word, expected):
    assert has_repeating_letters(word) is expected
    assert has_repeating_letters("spicy") is False


import argparse


def parse_args():
    parser = argparse.ArgumentParser(
        prog="wordle-manager",
        description="Examine and modify the Wordle word list",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    # Add flag for unique letters (no repeating letters within words)
    parser.add_argument(
        "-u",
        action="store_true",
        help="Only select words with unique letters (no repeating letters within each word)"
    )
    
    # Add optional flag for number of words
    parser.add_argument(
        "-n",
        type=int,
        default=3,
        dest="num_words",
        metavar="NUM",
        help="Number of words to generate (default: 3)"
    )

    subparsers = parser.add_subparsers(dest="action", required=False)
    subparsers.add_parser("stats", help="Show statistics")
    subparsers.add_parser("dedup", help="Remove duplicates")
    subparsers.add_parser("sort", help="Sort the list")
    subparsers.add_parser("clean", help="Remove invalid words")

    find_parser = subparsers.add_parser("find-scarce", help="Find scarce letters")
    find_parser.add_argument("--num", type=int, default=3)

    add_parser = subparsers.add_parser("add", help="Add a word")
    add_parser.add_argument("word", help="Word to add")
    args = parser.parse_args()
    return args



import pytest
from src.wordle_manager.cli import parse_args


@pytest.mark.parametrize(
    "sys_argv, expected_action",
    [
        (["wordle-manager", "stats"], "stats"),
        (["wordle-manager", "dedup"], "dedup"),
        (["wordle-manager", "sort"], "sort"),
        (["wordle-manager", "clean"], "clean"),
    ],
)
def test_parse_args_basic_actions(monkeypatch, sys_argv, expected_action):
    """Test parsing basic actions"""
    # Arrange
    monkeypatch.setattr("sys.argv", sys_argv)

    # Act
    args = parse_args()

    # Assert
    assert args.action == expected_action
    assert not hasattr(args, "num")
    assert not hasattr(args, "word")



import pytest
from unittest.mock import patch

from src.wordle_manager.utils import WordListManager
from src.wordle_manager import words


@pytest.fixture(autouse=True)
def safe_testing_environment():
    # Store the original word list
    original_words = words.word_list.copy()
    original_id = id(words.word_list)

    # Mock the save_to_file method to prevent any file writes
    with patch.object(WordListManager, "save_to_file", return_value=None) as mock_save:
        try:
            yield mock_save
        finally:
            # Restore to the original state
            if id(words.word_list) != original_id:
                words.word_list = original_words
            else:
                words.word_list[:] = original_words


class TestContextManagerExitAdditional:
    def test_exit_with_initial_state_none(self):
        # Arrange
        manager = WordListManager()
        manager._initial_state = None  # This could happen if __enter__ wasn't called

        # Act
        result = manager.__exit__(None, None, None)

        # Assert - handles None gracefully
        assert result is False

    def test_exit_exception_propagation(self):
        # Arrange
        manager = WordListManager()
        manager._initial_state = words.word_list.copy()

        # Act & Assert - __exit__ returns False, so exceptions should propagate
        with pytest.raises(ValueError):
            with manager:
                raise ValueError("This should not be suppressed")


class TestForTestingAdditionalCoverage:
    def test_empty_words_list(self):
        # Arrange
        words.word_list[:] = []

        # Act
        manager = WordListManager.for_testing()

        # Assert
        assert manager.word_list == []
        assert manager.word_list is not words.word_list
        assert isinstance(manager.word_list, list)


class TestFindScarceLetters:
    def test_find_scarce_letters_production_vs_test_mode(self, capsys):
        # Test mode case
        test_words = ["test", "mode"]
        test_manager = WordListManager.for_testing(test_words)
        test_manager.find_scarce_letters(num=1)
        test_output = capsys.readouterr().out

        # Production mode case - should use words.word_list
        prod_manager = WordListManager()
        prod_manager.find_scarce_letters(num=1)
        prod_output = capsys.readouterr().out

        # Outputs should be different, produce output, and do not crash
        assert test_output.strip() != ""
        assert prod_output.strip() != ""


class TestRemoveDuplicates:
    def test_remove_duplicates_production_vs_test_mode(self):
        # Arrange - add duplicates for production test
        words.word_list.extend(["test_dup", "test_dup"])

        # Test production mode - line 65: target_list = words.word_list (when not test_mode)
        prod_manager = WordListManager()
        original_prod_length = len(words.word_list)
        prod_manager.remove_duplicates()

        # Should have removed the duplicate 'test_dup'
        assert len(words.word_list) < original_prod_length

        # Test test mode - line 65: target_list = self.word_list (when test_mode)
        test_words = ["test_a", "test_a", "test_b"]
        test_manager = WordListManager.for_testing(test_words)
        test_manager.remove_duplicates()

        # Should only affect test_manager.word_list, not words.word_list
        assert test_manager.word_list == ["test_a", "test_b"]


class TestFileSafetyVerification:
    def test_save_to_file_mocked_no_actual_file_writes(self, safe_testing_environment):
        mock_save = safe_testing_environment

        # Arrange
        manager = WordListManager()
        words.word_list[:] = ["test", "data"]

        # Act - call a method that triggers save_to_file
        manager.sort_words()

        # Assert - save_to_file was called but mocked
        assert mock_save.called, "save_to_file should have been called"

    def test_add_word_in_production_mode_mocked(self, safe_testing_environment):
        mock_save = safe_testing_environment

        # Arrange - save_on_change=True by default
        manager = WordListManager()
        unique_word = "safetestword123"

        # Ensure word doesn't exist
        if unique_word in words.word_list:
            words.word_list.remove(unique_word)

        # Act
        result = manager.add_word(unique_word)

        # Assert
        assert result is True
        assert unique_word in words.word_list
        assert mock_save.called, "save_to_file should have been called but was mocked"


class TestShowStatsFormatting:
    def test_show_stats_total_words_formatting(self, capsys):
        # Arrange
        test_words = ["word1", "word2", "word3", "word4", "word5"]
        manager = WordListManager.for_testing(test_words)

        # Act
        manager.show_stats()

        # Assert formatting
        captured = capsys.readouterr()
        lines = captured.out.strip().split("\n")
        total_line = [line for line in lines if "Total words:" in line][0]

        # Right-aligned number with comma formatting
        assert "    5" in total_line
        assert "Total words:" in total_line


class TestRemoveInvalidWords:
    def test_remove_invalid_words_conditional_save_production_mode(self):
        # Arrange - production mode with mocked save_to_file
        words.word_list[:] = [
            "house",
            "inv@l",
            "xyz",
        ]
        with patch.object(WordListManager, "save_to_file"):
            manager = WordListManager()

            # Act
            manager.remove_invalid_words()

            # Assert - condition should be True
            assert manager.word_list == words.word_list

            # Verify invalid words were removed
            assert "inv@l" not in words.word_list  # Invalid: special character
            assert "xyz" not in words.word_list  # Invalid: no vowels
            assert "house" in words.word_list  # Valid: 5 letters, alpha, has vowels



from src.wordle_manager.words import word_list
from src.wordle_manager.utils import WordListManager
import pytest


@pytest.fixture
def word_list_manager():
    word_list_copy = word_list.copy()
    return WordListManager(word_list=word_list_copy, save_on_change=False)


def test_all_words_five_letters():
    assert all(len(word) == 5 for word in word_list)


def test_no_duplicate_words():
    assert len(word_list) == len(set(word_list))


def test_word_list_alphabetical():
    assert word_list == sorted(word_list)


def test_copy(word_list_manager):
    word_list_manager.add_word("abcde")
    assert "abcde" not in word_list
    assert "abcde" in word_list_manager.word_list


def test_add_word(word_list_manager):
    word_list_manager.add_word("bugaboo")
    assert "bugaboo" in word_list_manager.word_list


def test_add_existing_word(word_list_manager):
    result = word_list_manager.add_word("apple")
    assert result is False


def test_remove_invalid_words(word_list_manager):
    word_list_manager.remove_invalid_words()
    assert all(len(word) == 5 for word in word_list_manager.word_list)
    assert all(word.isalpha() for word in word_list_manager.word_list)
    assert all(any(char in "aeiouy" for char in word) for word in word_list_manager.word_list)



import os
from collections import Counter

from . import words

def has_repeating_letters(word):
    return len(set(word)) != len(word)

class WordListManager:
    def __init__(
        self, word_list: list[str] | None = None, save_on_change: bool = True
    ) -> None:
        if word_list is None:
            self.word_list = words.word_list
        else:
            self.word_list = word_list
        self.save_on_change = save_on_change
        self._initial_state = None

    def __enter__(self):
        """Context manager entry - save initial state"""
        self._initial_state = self.word_list.copy()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit - save changes if any were made"""
        if self.save_on_change:
            if self._initial_state != self.word_list:
                self.save_to_file()
                print("Changes saved automatically")
        return False

    @classmethod
    def for_testing(cls, test_word_list=None):
        """Create manager for testing that doesn't modify the original file"""
        if test_word_list is None:
            test_word_list = words.word_list.copy()
        return cls(word_list=test_word_list, save_on_change=False)

    def remove_invalid_words(self):
        VOWELS = set("aeiouy")
        original_count = len(self.word_list)
        valid_words = [
            word
            for word in self.word_list
            if len(word) == 5
            and word.isalpha()
            and any(char in VOWELS for char in word)
        ]
        removed_count = original_count - len(valid_words)

        if removed_count:
            print(f"Removed {removed_count} invalid words")
            self.word_list[:] = valid_words
            if self.save_on_change:
                self.save_to_file()
        else:
            print("No invalid words found")

    def find_scarce_letters(self, num=3):
        target_list = self.word_list
        letters_counts = Counter("".join(target_list))
        least_used_letters = letters_counts.most_common()[: -num - 1 : -1]
        for letter, count in least_used_letters:
            print(f"  {letter.upper()}: {count} occurrences")

    def remove_duplicates(self):
        target_list = self.word_list
        original_count = len(target_list)
        seen = set()
        target_list[:] = [
            word for word in target_list if not (word in seen or seen.add(word))
        ]
        removed_count = original_count - len(target_list)
        print(f"Removed {removed_count} duplicate words")
        print(f"Word list: {len(target_list)} unique words")
        if self.save_on_change:
            self.save_to_file()

    def sort_words(self):
        target_list = self.word_list
        target_list.sort()
        print("Word list sorted")
        if self.save_on_change:
            self.save_to_file()

    def add_word(self, word):
        target_list = self.word_list
        if word not in target_list:
            target_list.append(word)
            print(f"Added '{word}' to word list")
            if self.save_on_change:
                self.save_to_file()
            return True
        else:
            print(f"'{word}' already exists in word list")
            return False

    def save_to_file(self):  # pragma: no cover
        if not self.save_on_change:
            return
        script_dir = os.path.dirname(os.path.abspath(__file__))
        words_file = os.path.join(script_dir, "words.py")

        with open(words_file, "w") as f:
            f.write("word_list = [\n")
            for i, word in enumerate(words.word_list):
                if i % 10 == 0 and i > 0:
                    f.write("\n")
                f.write(f" '{word}',")
            f.write("\n]\n")

    def show_stats(self):
        target_list = self.word_list
        print("Word list statistics:")
        print(f"  Total words:    {len(target_list):>5,}")
        print(f"  Unique words:   {len(set(target_list)):>5,}")
        print(f"  Duplicates:     {len(target_list) - len(set(target_list)):>5,}")
        print(
            f"  List sorted:    {'Yes' if target_list == sorted(target_list) else 'No':>5}"
        )
